@@ -3,13 +3,13 @@
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
 r"""
 .. dialect:: postgresql+psycopg2
     :name: psycopg2
     :dbapi: psycopg2
     :connectstring: postgresql+psycopg2://user:password@host:port/dbname[?key=value&key=value...]
-    :url: http://pypi.python.org/pypi/psycopg2/
+    :url: https://pypi.org/project/psycopg2/
 
 psycopg2 Connect Arguments
 --------------------------
@@ -50,7 +50,7 @@ may be passed to :func:`_sa.create_engine()`, and include the following:
 
 * ``executemany_mode``, ``executemany_batch_page_size``,
   ``executemany_values_page_size``: Allows use of psycopg2
-  extensions for optimizing "executemany"-stye queries.  See the referenced
+  extensions for optimizing "executemany"-style queries.  See the referenced
   section below for details.
 
   .. seealso::
@@ -110,7 +110,7 @@ using ``host`` as an additional keyword argument::
 .. seealso::
 
     `PQconnectdbParams \
-    <http://www.postgresql.org/docs/9.1/static/libpq-connect.html#LIBPQ-PQCONNECTDBPARAMS>`_
+    <https://www.postgresql.org/docs/9.1/static/libpq-connect.html#LIBPQ-PQCONNECTDBPARAMS>`_
 
 .. _psycopg2_multi_host:
 
@@ -202,7 +202,7 @@ Psycopg2 Fast Execution Helpers
 
 Modern versions of psycopg2 include a feature known as
 `Fast Execution Helpers \
-<http://initd.org/psycopg/docs/extras.html#fast-execution-helpers>`_, which
+<https://initd.org/psycopg/docs/extras.html#fast-execution-helpers>`_, which
 have been shown in benchmarking to improve psycopg2's executemany()
 performance, primarily with INSERT statements, by multiple orders of magnitude.
 SQLAlchemy internally makes use of these extensions for ``executemany()`` style
@@ -596,7 +596,7 @@ class PGExecutionContext_psycopg2(PGExecutionContext):
 
     def create_server_side_cursor(self):
         # use server-side cursors:
-        # http://lists.initd.org/pipermail/psycopg/2007-January/005251.html
+        # https://lists.initd.org/pipermail/psycopg/2007-January/005251.html
         ident = "c_%s_%s" % (hex(id(self))[2:], hex(_server_side_id())[2:])
         return self._dbapi_connection.cursor(ident)
 
@@ -844,6 +844,25 @@ class PGDialect_psycopg2(PGDialect):
     def get_deferrable(self, connection):
         return connection.deferrable
 
+    def do_ping(self, dbapi_connection):
+        cursor = None
+        try:
+            dbapi_connection.autocommit = True
+            cursor = dbapi_connection.cursor()
+            try:
+                cursor.execute(self._dialect_specific_select_one)
+            finally:
+                cursor.close()
+                if not dbapi_connection.closed:
+                    dbapi_connection.autocommit = False
+        except self.dbapi.Error as err:
+            if self.is_disconnect(err, dbapi_connection, cursor):
+                return False
+            else:
+                raise
+        else:
+            return True
+
     def on_connect(self):
         extras = self._psycopg2_extras()
         extensions = self._psycopg2_extensions()
@@ -963,8 +982,8 @@ class PGDialect_psycopg2(PGDialect):
     @util.memoized_instancemethod
     def _hstore_oids(self, conn):
         extras = self._psycopg2_extras()
-        if hasattr(conn, "connection"):
-            conn = conn.connection
+        if hasattr(conn, "dbapi_connection"):
+            conn = conn.dbapi_connection
         oids = extras.HstoreAdapter.get_oids(conn)
         if oids is not None and oids[0]:
             return oids[0:2]
@@ -1018,7 +1037,7 @@ class PGDialect_psycopg2(PGDialect):
                 "connection not open",
                 "could not receive data from server",
                 "could not send data to server",
-                # psycopg2 client errors, psycopg2/conenction.h,
+                # psycopg2 client errors, psycopg2/connection.h,
                 # psycopg2/cursor.h
                 "connection already closed",
                 "cursor already closed",
